package nz.ac.massey.programming_project_159333_s1_2021;
/**
 * Component Array
 * Defines the data structure that component data is stored under.
 * Has a list of defined objects, and two associative maps that link
 * the position of the data with the entity number.
 * 
 * Therefore, every entity in entityComponentDataMap is valid, so
 * no additional sorting is required
 * 
 * 
 * Contributors:
 * Brychan Dempsey - brychand@hotmail.com
 * 
 */

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;


class ComponentArray{
    List<Object> componentArray = new ArrayList<>();

    Map<Integer, Integer> entityComponentDataMap = new HashMap<>();
    Map<Integer, Integer> componentDataEntityMap = new HashMap<>();

    public void entityDestroyed(int entity){
        if (entityComponentDataMap.containsKey(entity)){
            removeData(entity);
        }
    }

    public void removeData(int entity){
        if (!entityComponentDataMap.containsKey(entity)){
            System.err.println("Attempted to remove non-existent entity");
            return;
        }
        // Get the componentData index of the entity
        int removedComponentDataIndex = entityComponentDataMap.get(entity);
        // Replace the removed component with the last component in the array
        componentArray.set(removedComponentDataIndex, componentArray.get(componentArray.size() -1));
        // update the data positions in the map
        int lastEntity = componentDataEntityMap.get(componentArray.size()-1);
        entityComponentDataMap.replace(lastEntity, removedComponentDataIndex);
        componentDataEntityMap.replace(removedComponentDataIndex, lastEntity);
        // Finally, remomve the last elements
        entityComponentDataMap.remove(entity);
        componentDataEntityMap.remove(componentArray.size() -1);

        componentArray.remove(componentArray.size() -1);
        
    }

    public void insertData(int entity, Object component){
        if (entityComponentDataMap.containsKey(entity)){
            System.err.println("Entity is already subscribed to the component");
            return;
        }
        int index = componentArray.size();
        entityComponentDataMap.put(entity, index);
        componentDataEntityMap.put(index, entity);

        componentArray.add(component);
    }

    public Object getData(int entity){
        if (!entityComponentDataMap.containsKey(entity)){
            System.err.println("Attempted to retrieve non-existent data");
            return null;
        }

        return componentArray.get(entityComponentDataMap.get(entity));
    }
}