package nz.ac.massey.programming_project_159333_s1_2021;
/**
 * Entity Manager
 * This class manages entity allocations; keeping a list
 * of all unassigned entities values. 
 * Additionally, handles setting the registered component
 * flags for an entity.
 * 
 * Contributors:
 * Brychan Dempsey - brychand@hotmail.com
 * 
 */

import java.util.BitSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.ArrayList;

// Define the manager classes internally - should be moved to seperate source files as appropriate
/**
 * Manages data from the perspective of the entity.
 * <p>
 * I.e. Controls adding and removing entities, and registration and deregistration of components.
 */
class EntityManager{
    Queue<Integer> unusedEntities;
    List<BitSet> entityRegistrations;

    public EntityManager(){
        unusedEntities = new LinkedList<>();
        entityRegistrations = new ArrayList<>();

        for (int i = 0; i < 1024; i++) {
            unusedEntities.add(i);
            entityRegistrations.add(new BitSet());
        }
    }

    public EntityManager(int maxEntities){
        unusedEntities = new LinkedList<>();
        entityRegistrations = new ArrayList<>();
        
        for (int i = 0; i < maxEntities; i++) {
            unusedEntities.add(i);
            entityRegistrations.add(new BitSet());
        }
    }


    public Integer addEntity(){
        if (unusedEntities.size() == 0){
            System.err.println("No available space to create a new entity");
            return -1;
        }
        return unusedEntities.remove();
    }

    public void removeEntity(int entity){
        unusedEntities.add(entity);
        entityRegistrations.get(entity).clear();
    }

    public void registerComponent(int component, int entity){
        entityRegistrations.get(entity).set(component);
    }

    public void unregisterComponent(int component, int entity){
        entityRegistrations.get(entity).clear(component);
    }

    public BitSet getRegistrations(int entity){
        return entityRegistrations.get(entity);
    }
}